@@ -1,4 +1,5 @@
 import asyncio
+import os
 import inspect
 import json
 import logging
@@ -16,6 +17,10 @@ import aiohttp
 import aiohttp.web as aiohttp_web
 
 
+# 4MB is the default inside aiohttp
+MAX_MSG_SIZE = int(os.environ.get("WSLINK_MAX_MSG_SIZE", 4194304))
+
+
 async def _on_startup(app):
     # Emit an expected log message so launcher.py knows we've started up.
     logging.critical("wslink: Starting factory")
@@ -206,7 +211,7 @@ class WslinkHandler(object):
         aiohttp_app = request.app
 
         client_id = str(uuid.uuid4()).replace("-", "")
-        current_ws = aiohttp_web.WebSocketResponse()
+        current_ws = aiohttp_web.WebSocketResponse(max_msg_size=MAX_MSG_SIZE)
         self.connections[client_id] = current_ws
 
         logging.info("client {0} connected".format(client_id))
@@ -452,11 +457,12 @@ class WslinkHandler(object):
                 # decrement for key
                 pub.publishManager.unregisterAttachment(key)
 
-            pub.publishManager.freeAttachments(keys=found_keys)
-
         for ws in websockets:
             await ws.send_str(encMsg)
 
+        loop = asyncio.get_running_loop()
+        loop.call_soon(pub.publishManager.freeAttachments, found_keys)
+
     async def sendWrappedError(self, rpcid, code, message, data=None, client_id=None):
         wrapper = {
             "wslink": "1.0",
